﻿using CallCenter.BlackLists;
using CallCenter.Caches;
using CallCenter.Notifications;
using MediatR;
using CallCenter.Calls;
using NewRock.Sdk;
using NewRock.Sdk.Control.Request;
using CallCenter.Devices;
using CallCenter.Share.Enums;
using Microsoft.Extensions.Options;
using CallCenter.Tools;
using NewRock.Sdk.Accept.Request;
using CallCenter.Settings;
using NewRock.Sdk.Transfer.Connect.Request;
using XF.Domain.Constants;
using CallCenter.Ivrs;
using XF.Domain.Cache;
using Microsoft.Extensions.Logging;

namespace CallCenter.Application.Handlers
{
    public class InviteNotificationHandler : INotificationHandler<InviteNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly IUserCacheManager _userCacheManager;
        private readonly IBlacklistDomainService _blacklistDomainService;
        private readonly INewRockClient _newRockClient;
        private readonly IOptionsSnapshot<DeviceConfigs> _options;
        private readonly IOptionsSnapshot<WorkTimeSettings> _workTimeOptions;
        private readonly ISystemSettingCacheManager _systemSettingCacheManager;
        private readonly ITypedCache<WorkTimeSettings> _worktimeCache;
        private readonly ILogger<IncomingNotificationHandler> _logger;
        private readonly IIvrCacheManager _ivrCacheManager;

        public InviteNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository, IUserCacheManager userCacheManager, IBlacklistDomainService blacklistDomainService, INewRockClient newRockClient, IOptionsSnapshot<DeviceConfigs> options, IOptionsSnapshot<WorkTimeSettings> workTimeOptions, ISystemSettingCacheManager systemSettingCacheManager,ITypedCache<WorkTimeSettings> worktimeCache, IIvrCacheManager ivrCacheManager)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
            _userCacheManager = userCacheManager;
            _blacklistDomainService = blacklistDomainService;
            _newRockClient = newRockClient;
            _options = options;
            _workTimeOptions = workTimeOptions;
            _systemSettingCacheManager = systemSettingCacheManager;
            _worktimeCache = worktimeCache;
            _ivrCacheManager = ivrCacheManager;
        }

        public bool IsInWrokTime()
        {
            return false;
        }

        public async Task Handle(InviteNotification notification, CancellationToken cancellationToken)
        {
            //验证来电是否在黑名单中
            bool isblacklist = _blacklistDomainService.IsInBlacklist(notification.Visitor.From);
            //获取星期数
            //string weeekNum = WeekTool.GetWeekNum();

            //TODO 去掉绑定用户信息
            //var workModel = _userCacheManager.GetWorkByTel(notification.Visitor.To);
            var isp = PhoneIspTool.GetPhoneIsp(notification.Visitor.From);
            var model = new Call()
            {
                CallStatus = ECallStatus.PreIncoming,
                CallDirection = ECallDirection.In,
                CallType = ECallType.VisitorCallIn,
                ConversationId = notification.Visitor.Id,
                FromNo = notification.Visitor.From,
                ToNo = notification.Visitor.To,
                Trunk = notification.TrunkId,
                //UserId = workModel.UserId,
                //UserName = workModel.UserName,
                PhoneIsp = isp,
            };
            if (isblacklist)
                model.RingOffType = ERingOffType.BlackList;

            model.Modified();
            var callid = await _callRepository.AddAsync(model, cancellationToken);
            var detail = new CallDetail()
            {
                CallId = callid,
                CallStatus = ECallStatus.PreIncoming,
                OMCallId = notification.Visitor.CallId,
                ConversationId = notification.Visitor.Id,
                EventName = notification.Attribute,
                FromNo = notification.Visitor.From,
                ToNo = notification.Visitor.To,
            };
            await _callDetailRepository.AddAsync(detail, cancellationToken);

            //如果是在黑名单中直接挂断
            if (isblacklist)
            {
                await _newRockClient.ClearCall(new ClearCallRequest()
                {
                    Attribute = "Clear",
                    Visitor = new ClearCallVisitor()
                    {
                        Id = notification.Visitor.Id
                    }
                }, _options.Value.ReceiveKey, _options.Value.Expired, cancellationToken);
            }
            else
            {
                await _newRockClient.AcceptVisitor(
                    new AcceptVisitorRequest()
                    { Attribute = "Accept", Visitor = new AcceptVisitorModel() { Id = notification.Visitor.Id } },
                    _options.Value.ReceiveKey, _options.Value.Expired, cancellationToken);

                var setting = _systemSettingCacheManager.GetSetting(SettingConstants.IVRConfig);
                if (bool.Parse(setting.SettingValue))
                {
                    //TODO 获取工作或休息时间（接听策略）
                    //var ivrList = _ivrCacheManager.GetIvrs();
                    //var ivr = ivrList.First(x => x.IvrCategoryId == "08da9b9f-a35d-4ade-8ea7-55e8abbcdefd" && x.IsRoot);

                    var ivr = GetCorrectIvr();
                    _logger.LogInformation("transfer to ivr.no: {ivrNo}", ivr.No);
                    await _newRockClient.VisitorToMenu(
                        new VisitorToMenuRequest()
                        {
                            Attribute = "Connect",
                            Menu = new VisitorToMenuMenu() { Id = ivr.No },
                            Visitor = new VisitorToMenuVisitor() { Id = notification.Visitor.Id }
                        },
                        _options.Value.ReceiveKey, _options.Value.Expired, cancellationToken);
                }
                else
                {
                    //TODO 跳转默认分机组
                }
            }
        }


        #region private

        private Ivr GetCorrectIvr()
        {
            var worktimeSettings = _worktimeCache.GetOrAdd("worktimesettings", d => _workTimeOptions.Value, ExpireMode.Absolute, TimeSpan.FromDays(1));
            var categoryId = GetCorrectCategory(worktimeSettings);
            var ivrList = _ivrCacheManager.GetIvrs();
            var ivr = ivrList.First(x => x.IvrCategoryId == categoryId && x.IsRoot);
            return ivr;
        }

        private string GetCorrectCategory(WorkTimeSettings settings)
        {
            if (!settings.WorkDay.Contains((int)DateTime.Now.DayOfWeek))
                return settings.RestCategory;
            var time = TimeOnly.FromDateTime(DateTime.Now);
            if ((time >= TimeOnly.Parse(settings.MorningBegin) && time <= TimeOnly.Parse(settings.MorningEnd))
                || (time >= TimeOnly.Parse(settings.AfterBegin) && time <= TimeOnly.Parse(settings.AfterEnd))
                )
                return settings.WorkCategory;
            return settings.RestCategory;
        }

        #endregion
    }
}
